{ =====================================================================
  file     : proed4w.dpr
  by       : Jurjen Dijkstra, 2 March 2002
  compiler : Delphi 6
  purpose  : Launch tasks in a Progress session from "ED for Windows".
             The Progress session needs to have an initialized server
             socket, where proEd4Win.p runs persistent.
             If there is no such Progress session, proed4w.exe can launch one.
             see proed4w.htm for more information.
  ===================================================================== }
program proed4w;

{$APPTYPE CONSOLE}

uses
  SysUtils,
  Inifiles,
  windows,
  Sockets;

procedure ShowUsage;
begin
  writeln('');
  writeln('proed4w version 1 (2 March 2002)');
  writeln('usage:');
  writeln('  proed4w -H host -S port');
  writeln('');
  writeln('  -H host');
  writeln('     optional. Default host is 127.0.0.1');
  writeln('  -S portnumber');
  writeln('     optional. If -S is not specified then ');
  writeln('     portnumber will be read from file port.i');
  writeln('     which should reside in same directory as proed4w.exe.');
  writeln('     If port.i is not available then portnumber defaults to 2525.');
  writeln('  -cfg config');
  writeln('     optional. Launch a Progress session if no server is responding.');
  writeln('     Directory <proed4w>\settings\<config> specifies what to launch.');
  writeln('  /?');
  writeln('  -?');
  writeln('  /help');
  writeln('  -help');
  writeln('  -version');
  writeln('     optional. Shows this usage output.');
  writeln('');
  writeln('  All other parameters are forwarded to the server.');
  writeln('  All messages from the server are written to stdout.');
  writeln('  proed4w ends when server sends a <disconnect> message.');
  halt;
end;


procedure ShowUsageIfRequested;
var i     : integer;
    param : string;
begin
   for i:=1 to ParamCount do
      begin
        param:=ParamStr(i);
        if (param='/?') or
           (param='-?') or
           (param='-help') or
           (param='/help') or
           (param='-version') then
             begin
                ShowUsage;
                halt;
             end;
      end;
end;


function GetParameterValue (param : string) : string;
var prevparam : string;
    i         : integer;
begin
    prevparam := '';
    result    := '';
    for i:=1 to ParamCount do
      if prevparam=param then
         begin
           result := ParamStr(i);
           exit;
         end
      else
         prevparam := ParamStr(i);
end;

procedure MakeProcess(command, workingdir : PChar);
var si: STARTUPINFO;
    pi: PROCESS_INFORMATION;
    bResult  : LongBool;
begin
   fillchar (si,68,0);
   si.cb := 68;
   si.dwFlags := STARTF_USESHOWWINDOW;
   si.wShowWindow := SW_NORMAL;

   bResult := CreateProcess
     ( nil,
       command,
       nil,
       nil,
       FALSE,
       0,
       nil,
       workingdir,
       si,
       pi
     );

  if bResult then
    begin
      { Wait until the application is ready to accept messages. }
      { But wait no longer than 30 thousand milliseconds. That should do it}
      WaitForInputIdle(pi.hProcess, 30000);
      CloseHandle(pi.hProcess);
      CloseHandle(pi.hThread );
    end;

end;

procedure SpawnProgressServer;
var config, dir : string;
    inifile     : TInifile;
    command,
    params,
    workingdir  : string;
    p           : integer;
begin
    config := GetParameterValue('-cfg');
    if config='' then exit;

    { determine the directory where proed4w.ini is.
      this is a subdirectory of the dir where proed4w.exe is located. }
    dir := ExtractShortPathname(IncludeTrailingPathdelimiter(ExtractFiledir(ParamStr(0)))
           + 'settings\' + config);

    inifile    := TInifile.Create(dir + '\proed4w.ini');
    command    := inifile.ReadString('startup','command','');
    params     := inifile.ReadString('startup','params','');
    workingdir := inifile.ReadString('startup','workingdir','');
    inifile.Free;

    { params: replace each %dir% by dir}
    p := pos('%dir%', params);
    while p>0 do
      begin
        delete(params, p, length('%dir%'));
        insert(dir, params, p);
        p := pos('%dir%', params);
      end;

    if command<>'' then
       MakeProcess(PChar(command + ' ' + params), PChar(workingdir));
end;


function ReadFromPort_i : string;
var includefile : string;
    line        : string;
    F           : text;
begin
    { the default portnumber is specified on line 1 of file port.i }
    includefile := IncludeTrailingPathdelimiter(ExtractFiledir(ParamStr(0))) + 'port.i';
    AssignFile (F, includefile);
    try
      Reset(F);
      Readln(F, line);
      CloseFile(F);
    except
    end;
    result := trim(line);
end;

var
  ClientSocket : TTCPClient;
  Port         : string;
  Host         : string;
  line         : string;
  i            : integer;

begin
  ShowUsageIfRequested;

  ClientSocket := TTCPClient.Create(nil);

  Host := GetParameterValue('-H');
  if Host='' then Host:='127.0.0.1';

  Port := GetParameterValue('-S');
  if Port='' then
     Port := ReadFromPort_i;
  if Port='' then
     Port:='2525';

  ClientSocket.RemotePort := Port;
  ClientSocket.RemoteHost := Host;
  ClientSocket.BlockMode  := bmBlocking;

  { try to connect. If no server available then launch Progress }
  if not ClientSocket.Connect then
     if Host='127.0.0.1' then
        SpawnProgressServer;

  { try to connect again, to the newly spawned Progress server.
    If no success then just complain }
  if not ClientSocket.Connect then
     begin
       writeln('Can''t connect to a Progress session on port ' + Port);
       writeln('please make sure that proed4w.p is running.');
     end
  else
     begin

       { send commandline }
       line := '';
       for i:=1 to ParamCount do
         line := line + ' ' + ParamStr(i);
       if ClientSocket.Active then
          ClientSocket.SendLn(line, #0);

       { read answers from socket, write them to stdout }
       while ClientSocket.Active do
         try
            line := ClientSocket.ReceiveLn;
            if line='<disconnect>' then
               ClientSocket.Disconnect
            else
               if line<>'' then
                  writeln(line);
         except
            ClientSocket.Disconnect;
         end;

       { disconnect if still connected }
       if ClientSocket.Active then
          ClientSocket.Disconnect;

     end;

  { cleanup }
  ClientSocket.Free;
end.
