/* ============================================================================
   file    : proed4w.p
   by      : Jurjen Dijkstra
   purpose : communication with ED for Windows (the editor, www.getsoft.com).
   notes   : this procedure needs to be run persistent.
             see ..... for operation guidelines

             The server socket listens at port 2525.
             This is specified in includefile proed4w/port.i
             Proed4w.exe also reads this same port.i file.
             So if you want to change the portnumber, just change the
             contents of proed4w/port.i
   ============================================================================ */

DEFINE VARIABLE hServer      AS HANDLE NO-UNDO.
DEFINE VARIABLE hConnection  AS HANDLE NO-UNDO.

IF NOT THIS-PROCEDURE:PERSISTENT THEN DO:
   MESSAGE "procedure proed4w needs to be run persistent":T
           VIEW-AS ALERT-BOX.
   RUN CloseEd4WindowsServer.
   RETURN ERROR.
END.

/* there can be only one...
   destroy all previous instances, if any */
PUBLISH "CloseEd4WindowsServer":U.

CREATE SERVER-SOCKET hServer.
IF NOT hServer:ENABLE-CONNECTIONS("-S {proed4w/port.i}":U) THEN
   RUN CloseEd4WindowsServer. /* deletes this proc when server was already running */
ELSE
   hServer:SET-CONNECT-PROCEDURE("OnEd4WindowsConnect":U).

PUBLISH "Ed4WStateChanged":U ("waiting":U).

SUBSCRIBE TO "EndEd4WindowsConversation":U ANYWHERE.
SUBSCRIBE TO "WriteToEd4Windows":U ANYWHERE.
SUBSCRIBE TO "CloseEd4WindowsServer":U ANYWHERE.

RETURN.

ON CLOSE OF THIS-PROCEDURE
DO:
   RUN CloseEd4WindowsServer.
END.

/* make this procedure ADEPersistent */
{proed4w/_adetool.i}


PROCEDURE CloseEd4WindowsServer :
   /* purpose: delete this-procedure in a clean way. */

    IF VALID-HANDLE(hServer) THEN DO:
       hServer:DISABLE-CONNECTIONS().
       DELETE OBJECT hServer.
    END.
    RUN EndEd4WindowsConversation.
    DELETE PROCEDURE THIS-PROCEDURE.

END PROCEDURE.


PROCEDURE OnEd4WindowsConnect :
    /* purpose: handle an incoming connection from client (proed4w.exe) */
    DEFINE INPUT PARAMETER hpConnection AS HANDLE NO-UNDO.

    /* accept only one active connection at a time */
    IF VALID-HANDLE(hConnection) THEN
       IF hConnection:CONNECTED() THEN DO:
          hpConnection:DISCONNECT().
          RETURN.
       END.
       
    hConnection = hpConnection.
    hConnection:SET-READ-RESPONSE-PROCEDURE("ReadEd4WindowsCommand":U).

END PROCEDURE.


PROCEDURE ReadEd4WindowsCommand :
    /* purpose: proed4w.exe sends its own commandline to this socket,
                after that proed4w.exe will not send anything again and
                will just wait for answers. */
    DEFINE VARIABLE numbytes     AS INTEGER   NO-UNDO.
    DEFINE VARIABLE memory       AS MEMPTR    NO-UNDO.
    DEFINE VARIABLE commandline  AS CHARACTER NO-UNDO.

    /* ignore reads if not from accepted connection,
       although that could not possibly happen */
    IF SELF:HANDLE <> hConnection THEN
       RETURN.

    PUBLISH "Ed4WStateChanged":U ("working":U).
       
    /* read the TCP message. Assume it's one whole line, and one line only */
    numbytes = SELF:GET-BYTES-AVAILABLE().
    IF numbytes>0 THEN DO:
       SET-SIZE(memory) = numbytes + 1.
       SELF:READ(memory, 1, numbytes).
       PUT-BYTE(memory, numbytes + 1) = 0.
       commandline = GET-STRING(memory,1).
       SET-SIZE(memory) = 0.
    END.

    /* parse the commandline and do whatever you need to do */
    RUN ExecuteCommandline (commandline).

    /* finally, close the connection */
    RUN EndEd4WindowsConversation.

END PROCEDURE.


PROCEDURE ExecuteCommandline :
    /* purpose: figure out what proed4w.exe is asking and respond. */
    DEFINE INPUT PARAMETER commandline AS CHARACTER NO-UNDO.

    DEFINE VARIABLE ParamStr     as character no-undo.
    DEFINE VARIABLE PrevParamStr as character no-undo.
    define variable sourcefile   as character no-undo.
    define variable todo         as character no-undo.
    define variable i            as integer no-undo.

    PUBLISH "Ed4WCommandReceived":U (commandline).
    
    do i=1 to num-entries(commandline, " ":U) :
       PrevParamStr = ParamStr.
       ParamStr = entry(i, commandline, " ":U).
       if PrevParamStr = "-c":U then
          todo = ParamStr.
       if PrevParamStr = "-f":U then
          sourcefile = ParamStr.
    end.

    case todo:
      when "prolint":U then if search("prolint/prolint.p":U)<>? then
                               run prolint/prolint.p(sourcefile, ?, "ed4win":U, true).
                            else
                               run WriteToEd4Windows("prolint/prolint.p not found":T).

      WHEN "prolint-logwin":U THEN PUBLISH "Prolint_SendLogWin_Ed4Win":U.

      when "syntax":U  then run proed4w/syntaxcheck.p(sourcefile).

      when "close":U  then run CloseEd4WindowsServer.

      otherwise run WriteToEd4Windows(substitute("command '-c &1' not understood":T,todo)).

    end case.

END PROCEDURE.


PROCEDURE WriteToEd4Windows :
  /* purpose: write one line of text to Ed4Windows.
     Any procedure in this Progress session can do this by
     using the PUBLISH statement, for example:
        PUBLISH "WriteToEd4Windows" ("Hello Ed!").
  */
    DEFINE INPUT PARAMETER txt     AS CHARACTER NO-UNDO.

    DEFINE VARIABLE numbytes AS INTEGER NO-UNDO.
    DEFINE VARIABLE memory   AS MEMPTR  NO-UNDO.
    DEFINE VARIABLE written  AS INTEGER NO-UNDO.

    IF VALID-HANDLE(hConnection) THEN
       IF hConnection:CONNECTED() THEN DO:
          numbytes = LENGTH(txt).
          SET-SIZE(memory) = numbytes + 2.
          PUT-STRING(memory,1) = txt.

          /* make sure the string is CR/LF terminated. Proed4w.exe expects it! */
          PUT-BYTE(memory, numbytes + 1) = 13. /* CR */
          PUT-BYTE(memory, numbytes + 2) = 10. /* LF */

          written = 0.
          DO WHILE written<(numbytes + 2) :
             hConnection:WRITE (memory, 1 + written, numbytes + 2 - written).
             written = written + hConnection:BYTES-WRITTEN.
          END.

          SET-SIZE(memory) = 0.
       END.

END PROCEDURE.


PROCEDURE EndEd4WindowsConversation :
   /* purpose : disconnect proed4w.exe.

      Any procedure in this Progress session can do this by
      using the PUBLISH statement, for example:
         PUBLISH "EndEd4WindowsConversation".
   
      The current version of proed4w.exe has a problem:
      it waits forever for incoming messages and does not timeout
      in case the connection is lost. The workaround is to send
      proed4w.exe a "<disconnect>" string first. If you don't (or if
      the Progress session breaks or anything like that) proed4w.exe will
      wait forever! */

   IF VALID-HANDLE(hConnection) THEN
       IF hConnection:CONNECTED() THEN DO:
          RUN WriteToEd4Windows("<disconnect>":U).
          SELF:DISCONNECT().
   END.
   PUBLISH "Ed4WStateChanged":U ("waiting":U).

END PROCEDURE.


